@@ -67,6 +67,10 @@ Bank::~Bank()
 
 bool Bank::readOnlyInstrument(int ninstrument)
 {
+    if (readOnlyBank(currentBankID))
+        return true; // root and bank not writable
+    if (emptyslot(ninstrument)) // nothing there so must be OK to write
+        return false;
     const char * file = getFullPath(currentRootID, currentBankID, ninstrument).c_str();
     return access(file, W_OK);
 }
@@ -92,10 +96,10 @@ string Bank::getnamenumbered(unsigned int ninstrument)
 
 
 // Changes the name of an instrument (and the filename)
-void Bank::setname(unsigned int ninstrument, string newname, int newslot)
+bool Bank::setname(unsigned int ninstrument, string newname, int newslot)
 {
     if (emptyslot(ninstrument))
-        return;
+        return false;
 
     int slot = (newslot >= 0) ? newslot + 1 : ninstrument + 1;
     string filename = "0000" + asString(slot);
@@ -109,12 +113,14 @@ void Bank::setname(unsigned int ninstrument, string newname, int newslot)
     int chk = rename(getFullPath(currentRootID, currentBankID, ninstrument).c_str(), newfilepath.c_str());
     if (chk < 0)
     {
-        synth->getRuntime().Log("Error, bank setName failed renaming "
+        synth->getRuntime().Log("Bank: setName failed renaming "
                     + getFullPath(currentRootID, currentBankID, ninstrument) + " -> "
-                    + newfilepath + " : " + string(strerror(errno)));
+                    + newfilepath + ": " + string(strerror(errno)));
+        return false;
     }    
     instrRef.name = newname;
     instrRef.filename = filename;
+    return true;
 }
 
 
@@ -140,7 +146,7 @@ void Bank::clearslot(unsigned int ninstrument)
     int chk = remove(getFullPath(currentRootID, currentBankID, ninstrument).c_str());
     if (chk < 0)
     {
-        synth->getRuntime().Log("clearSlot " + asString(ninstrument) + " failed to remove "
+        synth->getRuntime().Log("clearSlot: " + asString(ninstrument) + " Failed to remove "
                      + getFullPath(currentRootID, currentBankID, ninstrument) + " "
                      + string(strerror(errno)));
     }
@@ -153,7 +159,7 @@ void Bank::savetoslot(unsigned int ninstrument, Part *part)
 {
     if (ninstrument >= BANK_SIZE)
     {
-        synth->getRuntime().Log("savetoslot " + asString(ninstrument) + ", slot > BANK_SIZE");
+        synth->getRuntime().Log("savetoslot: Saved " + asString(ninstrument) + ", slot > BANK_SIZE");
         return;
     }
     clearslot(ninstrument);
@@ -187,6 +193,69 @@ bool Bank::loadfromslot(unsigned int ninstrument, Part *part)
 }
 
 
+bool Bank::readOnlyBank(int bankID)
+{
+    const char * root = getRootPath(currentRootID).c_str();
+    if (access(root, W_OK)) // root not writable
+        return true;
+    if (getBankName(bankID).empty())
+        return false; // not there so must be OK to write
+    const char * file = getBankPath(currentRootID, bankID).c_str();
+    return access(file, W_OK);
+}
+
+
+//Gets a bank name
+string Bank::getBankName(int bankID)
+{
+    if (roots [currentRootID].banks.count(bankID) == 0)
+        return "";
+    return string(roots [currentRootID].banks [bankID].dirname);
+}
+
+
+//Gets a bank name with ID
+string Bank::getBankIDname(int bankID)
+{
+    string retname = getBankName(bankID);
+    if (!retname.empty())
+        retname = asString(bankID) + ". " + retname;
+    return retname;
+}
+
+
+// finds the number of instruments in a bank
+int Bank::getBankSize(int bankID)
+{
+    int found = 0;
+    for (int i = 0; i < BANK_SIZE; ++ i)
+        if (!roots [currentRootID].banks [bankID].instruments [i].name.empty())
+            found += 1;
+    return found;
+}
+
+// Changes a bank name 'in place' and updates the filename
+bool Bank::setbankname(unsigned int bankID, string newname)
+{
+    string filename = newname;
+    legit_filename(filename);
+    string newfilepath = getRootPath(currentRootID) + "/" + filename;
+    int chk = rename(getBankPath(currentRootID,bankID).c_str(),
+                     newfilepath.c_str());
+    if (chk < 0)
+    {
+        synth->getRuntime().Log("Bank: Failed to rename " + getBankName(bankID)
+                               + " to " + newname);
+        return false;
+    }
+    synth->getRuntime().Log("Renaming " + getBankName(bankID)
+                               + " to " + newname);
+    
+    roots [currentRootID].banks [bankID].dirname = newname;
+    return true;
+}
+
+
 // Makes current a bank directory
 bool Bank::loadbank(size_t rootID, size_t banknum)
 {
@@ -257,11 +326,32 @@ bool Bank::loadbank(size_t rootID, size_t banknum)
     return true;
 }
 
-
+// no longer used.
 // Makes a new bank, put it on a file and makes it current bank
-bool Bank::newbank(string newbankdir)
+/*bool Bank::newbank(string newbankdir)
 {
-    if (getRootPath(currentRootID).empty())
+    if (!newbankfile(newbankdir))
+        return false;
+    currentBankID = add_bank(newbankdir, newbankdir, currentRootID);
+    return true;
+}*/
+
+
+// Makes a new bank with known ID. Does *not* make it current
+bool Bank::newIDbank(string newbankdir, unsigned int bankID)
+{
+    if (!newbankfile(newbankdir))
+        return false;
+    roots [currentRootID].banks [bankID].dirname = newbankdir;
+    hints [currentRootID] [newbankdir] = bankID; // why do we need this?
+    return true;
+}
+
+
+// Performs the actual file operation for new banks
+bool Bank::newbankfile(string newbankdir)
+{
+     if (getRootPath(currentRootID).empty())
     {
         synth->getRuntime().Log("Current bank root directory not set");
         return false;
@@ -273,7 +363,7 @@ bool Bank::newbank(string newbankdir)
     int result = mkdir(newbankpath.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
     if (result < 0)
     {
-        synth->getRuntime().Log("Error, failed to mkdir " + newbankpath);
+        synth->getRuntime().Log("Bank: Failed to mkdir " + newbankpath);
         return false;
     }
     else
@@ -284,21 +374,56 @@ bool Bank::newbank(string newbankdir)
     forcefile += force_bank_dir_file;
     FILE *tmpfile = fopen(forcefile.c_str(), "w+");
     fclose(tmpfile);
-    currentBankID = add_bank(newbankdir, newbankdir, currentRootID);
     return true;
 }
 
 
+// Removes a bank and all its contents
+bool Bank::removebank(unsigned int bankID)
+{
+    int chk;
+
+    for (int inst = 0; inst < BANK_SIZE; ++ inst)
+        if (!roots [currentRootID].banks [bankID].instruments [inst].name.empty())
+        {
+            chk = remove(getFullPath(currentRootID, bankID, inst).c_str());
+            if (chk < 0)
+            {
+                synth->getRuntime().Log("removebank: "
+                                        + asString(inst) + " Failed to remove "
+                                        + getFullPath(currentRootID, bankID, inst) + " "
+                                        + string(strerror(errno)));
+                return false;
+            }
+            deletefrombank(currentRootID, bankID, inst);
+        }
+        string tmp = getBankPath(currentRootID, bankID)+"/.bankdir";
+        chk = remove(tmp.c_str());
+        if (chk < 0)
+        {
+            synth->getRuntime().Log("removebank: Failed to remove bank ID file"
+                                    + string(strerror(errno)));
+            return false;
+        }
+        chk = remove(getBankPath(currentRootID, bankID).c_str());
+        if (chk < 0)
+        {
+            synth->getRuntime().Log("removebank: Failed to remove bank"
+                                    + asString(bankID) + ": "
+                                    + string(strerror(errno)));
+            return false;
+        }
+        roots [currentRootID].banks.erase(bankID);
+   return true;
+}
+
+
 // Swaps a slot with another
 void Bank::swapslot(unsigned int n1, unsigned int n2)
 {
     if (n1 == n2)
         return;
-    if (locked())
-    {
-        synth->getRuntime().Log("swapslot requested, but is locked");
-        return;
-    }
+
     if (emptyslot(n1) && emptyslot(n2))
         return;
     if (emptyslot(n1)) // make the empty slot the destination
@@ -331,6 +456,68 @@ void Bank::swapslot(unsigned int n1, unsigned int n2)
 }
 
 
+// Intelligently moves or swaps banks preserving instrument details
+void Bank::swapbanks(unsigned int firstID, unsigned int secondID)
+{
+    if (firstID == secondID)
+    {
+        synth->getRuntime().Log("Nothing to move!");
+        return;
+    }
+        
+    string firstname = getBankName(firstID); // this needs improving
+    string secondname = getBankName(secondID);
+    if (firstname.empty() and secondname.empty())
+    {
+        synth->getRuntime().Log("Nothing to move!");
+        return;
+    }
+    if (secondname.empty())
+    {
+        synth->getRuntime().Log("Moving " + firstname);
+        roots [currentRootID].banks [secondID] = roots [currentRootID].banks [firstID];
+        roots [currentRootID].banks.erase(firstID);
+    }
+    else if (firstname.empty())
+    {
+        synth->getRuntime().Log("Moving " + secondname);
+        roots [currentRootID].banks [firstID] = roots [currentRootID].banks [secondID];
+        roots [currentRootID].banks.erase(secondID);
+    }
+    else
+    {
+        synth->getRuntime().Log("Swapping " + firstname + " with " + secondname );
+        roots [currentRootID].banks [firstID].dirname = secondname;
+        roots [currentRootID].banks [secondID].dirname = firstname;
+        hints [currentRootID] [secondname] = firstID; // why do we need these?
+        hints [currentRootID] [firstname] = secondID;
+
+        for(int pos = 0; pos < BANK_SIZE; ++ pos)
+        {
+            InstrumentEntry &instrRef_1 = getInstrumentReference(currentRootID, firstID, pos);
+            InstrumentEntry &instrRef_2 = getInstrumentReference(currentRootID, secondID, pos);
+
+            InstrumentEntry tmp = instrRef_2;
+            
+            if (instrRef_1.name == "")
+                roots [currentRootID].banks [secondID].instruments.erase(pos);
+            else
+                instrRef_2 = instrRef_1;
+            
+            if (tmp.name == "")
+                roots [currentRootID].banks [firstID].instruments.erase(pos);
+            else
+                instrRef_1 = tmp;
+        }
+    }
+    
+    if (firstID == currentBankID)
+        currentBankID = secondID;
+    else if(secondID == currentBankID)
+        currentBankID = firstID;
+}
+
+
 // Re-scan for directories containing instrument banks
 void Bank::rescanforbanks(void)
 {    
@@ -354,7 +541,7 @@ void Bank::scanrootdir(int root_idx)
     DIR *dir = opendir(rootdir.c_str());
     if (dir == NULL)
     {
-        synth->getRuntime().Log("No such directory, root bank entry: " + rootdir);
+        synth->getRuntime().Log("No such directory, root bank entry " + rootdir);
         return;
     }
     struct dirent *fn;
@@ -377,7 +564,7 @@ void Bank::scanrootdir(int root_idx)
         DIR *d = opendir(chkdir.c_str());
         if (d == NULL)
         {
-            synth->getRuntime().Log("Failed to open bank directory candidate: " + chkdir);
+            synth->getRuntime().Log("scanrootdir: Failed to open bank directory candidate " + chkdir);
             continue;
         }
         struct dirent *fname;
@@ -393,33 +580,17 @@ void Bank::scanrootdir(int root_idx)
                 bankDirsMap [candidate] = chkdir;
                 break;
             }
-
-// we don't need these checks as non-prefixed instruments are valid.
-//# warning this needs tidying up
-/*            if (possible.size() <= (xizext.size() + 5))
-                continue;
-            // check for an instrument starting with "NNNN-" prefix
-            for (idx = 0; idx < 4; ++idx)
+            string chkpath = chkdir + "/" + possible;
+            lstat(chkpath.c_str(), &st);
+            if (st.st_mode & (S_IFREG | S_IRGRP))
             {
-                x = possible.at(idx);
-                if (x < '0' || x > '9')
-                    break;
-            }
-            if (idx < 4 || possible.at(idx) != '-')
-                continue;*/
-            {
-                string chkpath = chkdir + "/" + possible;
-                lstat(chkpath.c_str(), &st);
-                if (st.st_mode & (S_IFREG | S_IRGRP))
+                // check for .xiz extension
+                if ((xizpos = possible.rfind(xizext)) != string::npos)
                 {
-                    // check for .xiz extension
-                    if ((xizpos = possible.rfind(xizext)) != string::npos)
-                    {
-                        if (xizext.size() == (possible.size() - xizpos))
-                        {   // is an instrument, so add the bank
-                            bankDirsMap [candidate] = chkdir;
-                            break;
-                        }
+                    if (xizext.size() == (possible.size() - xizpos))
+                    {   // is an instrument, so add the bank
+                        bankDirsMap [candidate] = chkdir;
+                        break;
                     }
                 }
             }
@@ -483,13 +654,17 @@ bool Bank::addtobank(size_t rootID, size_t bankID, int pos, const string filenam
     instrRef.name = name;
     instrRef.filename = filename;
     instrRef.PADsynth_used = false;
+    instrRef.ADDsynth_used = false;
+    instrRef.SUBsynth_used = false;
 
-    // see if PADsynth is used
-    if (synth->getRuntime().CheckPADsynth)
+    // see which engines are used
+    if (synth->getRuntime().checksynthengines)
     {
         XMLwrapper *xml = new XMLwrapper(synth);
         xml->checkfileinformation(getFullPath(rootID, bankID, pos));
         instrRef.PADsynth_used = xml->information.PADsynth_used;
+        instrRef.ADDsynth_used = xml->information.ADDsynth_used;
+        instrRef.SUBsynth_used = xml->information.SUBsynth_used;
         delete xml;
     }
     return 0;
@@ -531,6 +706,7 @@ size_t Bank::add_bank(string name, string , size_t rootID)
     return newIndex;
 }
 
+
 InstrumentEntry &Bank::getInstrumentReference(size_t ninstrument)
 {
     return getInstrumentReference(currentRootID, currentBankID, ninstrument);
@@ -550,12 +726,15 @@ void Bank::addDefaultRootDirs()
         "/usr/local/share/zynaddsubfx/banks",
         string(getenv("HOME")) + "/banks",
         "../banks",
-        "banks"
+        "end"
     };
-    for (unsigned int i = 0; i < (sizeof(bankdirs) / sizeof(bankdirs [0])); ++i)
+    int i = 0;
+    while (bankdirs [i] != "end")
     {
         addRootDir(bankdirs [i]);
+        ++ i;
     }
+    rescanforbanks();
 }
 
 
@@ -679,9 +858,12 @@ const BankEntry &Bank::getBank(size_t bankID)
 }
 
 
-bool Bank::isPADsynth_used(unsigned int ninstrument)
+int Bank::engines_used(unsigned int ninstrument)
 {
-    return getInstrumentReference(ninstrument).PADsynth_used;
+    int tmp = getInstrumentReference(ninstrument).ADDsynth_used
+            | (getInstrumentReference(ninstrument).SUBsynth_used << 1)
+            | (getInstrumentReference(ninstrument).PADsynth_used << 2);
+    return tmp;
 }
 
 
@@ -698,7 +880,7 @@ void Bank::removeRoot(size_t rootID)
         currentRootID = 0;
     }
     roots.erase(rootID);
-
+    setCurrentRootID(currentRootID);
 }
 
 
@@ -761,8 +943,8 @@ bool Bank::setCurrentBankID(size_t newBankID, bool ignoreMissing)
 
 size_t Bank::addRootDir(string newRootDir)
 {
-
-    if(newRootDir.empty() || !isDirectory(newRootDir))
+   // we need the size check to prevent weird behaviour if the name is just ./
+    if(!isDirectory(newRootDir) || newRootDir.length() < 4)
     {
         return 0;
     }
