@@ -47,36 +47,41 @@ using namespace std;
 #include "Misc/XMLwrapper.h"
 #include "Misc/SynthEngine.h"
 #include "Misc/Config.h"
+#include "MasterUI.h"
 
+extern void mainRegisterAudioPort(SynthEngine *s, int portnum);
 
 const unsigned short Config::MaxParamsHistory = 25;
 
 static char prog_doc[] =
     "Yoshimi " YOSHIMI_VERSION ", a derivative of ZynAddSubFX - "
     "Copyright 2002-2009 Nasca Octavian Paul and others, "
-    "Copyright 2009-2011 Alan Calvert";
+    "Copyright 2009-2011 Alan Calvert, "
+    "Copyright 20012-2013 Jeremy Jongepier and others, "
+    "Copyright 20014-2015 Will Godfrey and others";
 
 const char* argp_program_version = "Yoshimi " YOSHIMI_VERSION;
 
 static struct argp_option cmd_options[] = {
-    {"alsa-audio",        'A',  "<device>", 0x1,  "use alsa audio output" },
-    {"alsa-midi",         'a',  "<device>", 0x1,  "use alsa midi input" },
-    {"buffersize",        'b',  "<size>",     0,  "set alsa audio buffer size" },
+    {"alsa-audio",        'A',  "<device>",   1,  "use alsa audio output" },
+    {"alsa-midi",         'a',  "<device>",   1,  "use alsa midi input" },
+    {"define-root",       'D',  "<path>",     0,  "define path to new bank root"},
+    {"buffersize",        'b',  "<size>",     0,  "set internal buffer size" },
     {"show-console",      'c',  NULL,         0,  "show console on startup" },
     {"no-gui",            'i',  NULL,         0,  "no gui"},
-    {"jack-audio",        'J',  "<server>", 0x1,  "use jack audio output" },
-    {"jack-midi",         'j',  "<device>", 0x1,  "use jack midi input" },
+    {"jack-audio",        'J',  "<server>",   1,  "use jack audio output" },
+    {"jack-midi",         'j',  "<device>",   1,  "use jack midi input" },
     {"autostart-jack",    'k',  NULL,         0,  "auto start jack server" },
     {"auto-connect",      'K',  NULL,         0,  "auto connect jack audio" },
     {"load",              'l',  "<file>",     0,  "load .xmz file" },
     {"load-instrument",   'L',  "<file>",     0,  "load .xiz file" },
     {"name-tag",          'N',  "<tag>",      0,  "add tag to clientname" },
     {"samplerate",        'R',  "<rate>",     0,  "set alsa audio sample rate" },
-    {"oscilsize",         'o',  "<size>",     0,  "set oscilsize" },
-    {"state",             'S',  "<file>",   0x1,  "load state from <file>, defaults to '$HOME/.config/yoshimi/yoshimi.state'" },
+    {"oscilsize",         'o',  "<size>",     0,  "set AddSynth oscilator size" },
+    {"state",             'S',  "<file>",     1,  "load saved state, defaults to '$HOME/.config/yoshimi/yoshimi.state'" },
     #if defined(JACK_SESSION)
         {"jack-session-uuid", 'U',  "<uuid>",     0,  "jack session uuid" },
-        {"jack-session-file", 'u',  "<file>",     0,  "load jack session file" },
+        {"jack-session-file", 'u',  "<file>",     0,  "load named jack session file" },
     #endif
     { 0, }
 };
@@ -101,17 +106,21 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     connectJackaudio(false),
     alsaAudioDevice("default"),
     alsaMidiDevice("default"),
-    BankUIAutoClose(0),
     GzipCompression(3),
     Interpolation(0),
-    CheckPADsynth(1),
-    EnableProgChange(0), // default will be inverted
+    checksynthengines(1),
+    EnableProgChange(1), // default will be inverted
+    consoleMenuItem(0),
     rtprio(50),
-    midi_bank_root(128),
-    midi_bank_C(0),
+    midi_bank_root(0), // 128 is used as 'disabled'
+    midi_bank_C(32),
     midi_upper_voice_C(128),
-    enable_part_on_voice_load(0),
-    single_row_panel(0),
+    enable_part_on_voice_load(1),
+    single_row_panel(1),
+    NumAvailableParts(NUM_MIDI_CHANNELS),
+    nrpnL(127),
+    nrpnH(127),
+    nrpnActive(false),
     deadObjects(NULL),
     nextHistoryIndex(numeric_limits<unsigned int>::max()),
     sigIntActive(0),
@@ -148,8 +157,10 @@ bool Config::Setup(int argc, char **argv)
     switch (audioEngine)
     {
         case alsa_audio:
+        {
             audioDevice = string(alsaAudioDevice);
             break;
+        }
         case jack_audio:
             audioDevice = string(jackServer);
             break;
@@ -190,7 +201,7 @@ bool Config::Setup(int argc, char **argv)
         if (! isRegFile(StateFile))
         {
             no_state1: delete (fp);
-            no_state0: Log("Invalid state file specified for restore: " + StateFile);
+            no_state0: Log("Invalid state file specified for restore " + StateFile);
             return false;
         }
     }    
@@ -269,20 +280,14 @@ string Config::testCCvalue(int cc)
         case 1:
             result = "mod wheel";
             break;
-        case 7:
-            result = "volume";
-            break;
         case 10:
             result = "panning";
             break;
         case 11:
             result = "expression";
             break;
-        case 64:
-            result = "sustain pedal";
-            break;
-        case 65:
-            result = "partamento";
+        case 38:
+            result = "data lsb";
             break;
         case 71:
             result = "filter Q";
@@ -302,6 +307,45 @@ string Config::testCCvalue(int cc)
         case 78:
             result = "resonance bandwidth";
             break;
+        default:
+            result = masterCCtest(cc);
+    }
+    return result;
+}
+
+
+string Config::masterCCtest(int cc)
+{
+    string result = "";
+    switch (cc)
+    {
+         case 6:
+            result = "data msb";
+            break;
+        case 7:
+            result = "volume";
+            break;
+        case 38:
+            result = "data lsb";
+            break;
+        case 64:
+            result = "sustain pedal";
+            break;
+        case 65:
+            result = "portamento";
+            break;
+        case 96:
+            result = "data increment";
+            break;
+        case 97:
+            result = "data decrement";
+            break;
+        case 98:
+            result = "NRPN lsb";
+            break;
+        case 99:
+            result = "NRPN msb";
+            break;
         case 120:
             result = "all sounds off";
             break;
@@ -315,11 +359,11 @@ string Config::testCCvalue(int cc)
         {
             if (cc < 128) // don't compare with 'disabled' state
             {
-                if (cc == synth->getRuntime().midi_bank_C)
+                if (cc == midi_bank_C)
                     result = "bank change";
-                else if (cc == synth->getRuntime().midi_bank_root)
+                else if (cc == midi_bank_root)
                     result = "bank root change";
-                else if (cc == synth->getRuntime().midi_upper_voice_C)
+                else if (cc == midi_upper_voice_C)
                     result = "extended program change";
             }
         }
@@ -330,7 +374,7 @@ string Config::testCCvalue(int cc)
 
 void Config::clearPresetsDirlist(void)
 {
-    for (int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
+    for (int i = 0; i < MAX_PRESETS; ++i)
         presetsDirlist[i].clear();
 }
 
@@ -362,10 +406,17 @@ bool Config::loadConfig(void)
     if (!isRegFile(resConfigFile) && !isRegFile(ConfigFile))
     {
         Log("ConfigFile " + resConfigFile + " not found");
+        Log("Trying for old config file");
         string oldConfigFile = string(getenv("HOME")) + string("/.yoshimiXML.cfg");
+        if (!isRegFile(oldConfigFile))
+        {
+            Log("Old config file " + resConfigFile + " not found");
+            Log("Trying for ZynAddSubFX config file");
+            oldConfigFile = string(getenv("HOME")) + string("/.zynaddsubfxXML.cfg");
+        }
         if (isRegFile(oldConfigFile))
         {
-            Log("Copying old config file " + oldConfigFile + " to new location: " + resConfigFile);
+            Log("Copying old config file " + oldConfigFile + " to new location " + resConfigFile);
             FILE *oldfle = fopen (oldConfigFile.c_str(), "r");
             FILE *newfle = fopen (resConfigFile.c_str(), "w");
             if (oldfle != NULL && newfle != NULL)
@@ -382,7 +433,10 @@ bool Config::loadConfig(void)
 
     bool isok = true;
     if (!isRegFile(resConfigFile) && !isRegFile(ConfigFile))
+    {
         Log("ConfigFile " + resConfigFile + " still not found, will use default settings");
+        saveConfig();
+    }
     else
     {
         XMLwrapper *xml = new XMLwrapper(synth);
@@ -421,15 +475,14 @@ bool Config::extractConfigData(XMLwrapper *xml)
         return false;
     }
     Samplerate = xml->getpar("sample_rate", Samplerate, 44100, 96000);
-    Buffersize = xml->getpar("sound_buffer_size", Buffersize, 64, 4096);
+    Buffersize = xml->getpar("sound_buffer_size", Buffersize, 32, 1024);
     Oscilsize = xml->getpar("oscil_size", Oscilsize,
                                         MAX_AD_HARMONICS * 2, 131072);
-    BankUIAutoClose = xml->getpar("bank_window_auto_close",
-                                               BankUIAutoClose, 0, 1);
     GzipCompression = xml->getpar("gzip_compression", GzipCompression, 0, 9);
     Interpolation = xml->getpar("interpolation", Interpolation, 0, 1);
-    CheckPADsynth = xml->getpar("check_pad_synth", CheckPADsynth, 0, 1);
+    checksynthengines = xml->getpar("check_pad_synth", checksynthengines, 0, 1);
     EnableProgChange = 1 - xml->getpar("ignore_program_change", EnableProgChange, 0, 1); // inverted for Zyn compatibility
+    consoleMenuItem = xml->getpar("reports_destination", consoleMenuItem, 0, 1);
     VirKeybLayout = xml->getpar("virtual_keyboard_layout", VirKeybLayout, 0, 10);
 
     // get bank dirs
@@ -437,7 +490,7 @@ bool Config::extractConfigData(XMLwrapper *xml)
 
     // get preset dirs
     int count = 0;
-    for (int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
+    for (int i = 0; i < MAX_PRESETS; ++i)
     {
         if (xml->enterbranch("PRESETSROOT", i))
         {
@@ -476,6 +529,7 @@ bool Config::extractConfigData(XMLwrapper *xml)
     midi_bank_C = xml->getpar("midi_bank_C", midi_bank_C, 0, 128);
     midi_upper_voice_C = xml->getpar("midi_upper_voice_C", midi_upper_voice_C, 0, 128);
     enable_part_on_voice_load = xml->getpar("enable_part_on_voice_load", enable_part_on_voice_load, 0, 1);
+    consoleMenuItem = xml->getpar("enable_console_window", consoleMenuItem, 0, 1);
     single_row_panel = xml->getpar("single_row_panel", single_row_panel, 0, 1);
 
     if (xml->enterbranch("XMZ_HISTORY"))
@@ -535,16 +589,16 @@ void Config::addConfigXML(XMLwrapper *xmltree)
     xmltree->addpar("sample_rate", Samplerate);
     xmltree->addpar("sound_buffer_size", Buffersize);
     xmltree->addpar("oscil_size", Oscilsize);
-    xmltree->addpar("bank_window_auto_close", BankUIAutoClose);
 
     xmltree->addpar("gzip_compression", GzipCompression);
-    xmltree->addpar("check_pad_synth", CheckPADsynth);
-    xmltree->addpar("ignore_program_change", (1 - EnableProgChange));    
+    xmltree->addpar("check_pad_synth", checksynthengines);
+    xmltree->addpar("ignore_program_change", (1 - EnableProgChange));
+    xmltree->addpar("reports_destination", consoleMenuItem);
     xmltree->addpar("virtual_keyboard_layout", VirKeybLayout);
 
     synth->getBankRef().saveToConfigFile(xmltree);
 
-    for (int i = 0; i < MAX_BANK_ROOT_DIRS; ++i)
+    for (int i = 0; i < MAX_PRESETS; ++i)
         if (presetsDirlist[i].size())
         {
             xmltree->beginbranch("PRESETSROOT",i);
@@ -561,6 +615,7 @@ void Config::addConfigXML(XMLwrapper *xmltree)
     xmltree->addpar("midi_bank_C", midi_bank_C);
     xmltree->addpar("midi_upper_voice_C", midi_upper_voice_C);
     xmltree->addpar("enable_part_on_voice_load", enable_part_on_voice_load);
+    xmltree->addpar("enable_console_window", consoleMenuItem);
     xmltree->addpar("single_row_panel", single_row_panel);
 
     // Parameters history
@@ -662,8 +717,15 @@ void Config::addRuntimeXML(XMLwrapper *xml)
 
 void Config::Log(string msg, bool tostderr)
 {
-    if (showGui && !tostderr)
-        LogList.push_back(msg);
+    int pos;
+    if (showGui && !tostderr && consoleMenuItem)
+    {
+        pos = msg.find(":");
+        if (pos > 1)
+            LogList.push_back(msg.substr(pos + 2));
+        else
+            LogList.push_back(msg);
+    }
     else
         cerr << msg << endl;
 }
@@ -671,12 +733,9 @@ void Config::Log(string msg, bool tostderr)
 
 void Config::StartupReport(MusicClient *musicClient)
 {
-    if (!showGui)
-        return;
-
     Log(string(argp_program_version));
-    Log("Clientname: " + musicClient->midiClientName());
-    string report = "Audio: ";
+    Log("Config: Clientname: " + musicClient->midiClientName());
+    string report = "Config: Audio: ";
     switch (audioEngine)
     {
         case jack_audio:
@@ -690,7 +749,7 @@ void Config::StartupReport(MusicClient *musicClient)
     }
     report += (" -> '" + audioDevice + "'");
     Log(report);
-    report = "Midi: ";
+    report = "Config: Midi: ";
     switch (midiEngine)
     {
         case jack_midi:
@@ -706,9 +765,9 @@ void Config::StartupReport(MusicClient *musicClient)
         midiDevice = "default";
     report += (" -> '" + midiDevice + "'");
     Log(report);
-    Log("Oscilsize: " + asString(synth->oscilsize));
-    Log("Samplerate: " + asString(synth->samplerate));
-    Log("Period size: " + asString(synth->buffersize));
+    Log("Config: Oscilsize: " + asString(synth->oscilsize));
+    Log("Config: Samplerate: " + asString(synth->samplerate));
+    Log("Config: Period size: " + asString(synth->buffersize));
 }
 
 
@@ -741,7 +800,7 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                 {
                     if ((chk = pthread_attr_setschedpolicy(&attr, SCHED_FIFO)))
                     {
-                        Log("Failed to set SCHED_FIFO policy in thread attribute: "
+                        Log("Failed to set SCHED_FIFO policy in thread attribute "
                                     + string(strerror(errno))
                                     + " (" + asString(chk) + ")", true);
                         schedfifo = false;
@@ -749,7 +808,7 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                     }
                     if ((chk = pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED)))
                     {
-                        Log("Failed to set inherit scheduler thread attribute: "
+                        Log("Failed to set inherit scheduler thread attribute "
                                     + string(strerror(errno)) + " ("
                                     + asString(chk) + ")", true);
                         schedfifo = false;
@@ -762,7 +821,7 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                     prio_params.sched_priority = (prio > 0) ? prio : 0;
                     if ((chk = pthread_attr_setschedparam(&attr, &prio_params)))
                     {
-                        Log("Failed to set thread priority attribute: ("
+                        Log("Failed to set thread priority attribute ("
                                     + asString(chk) + ")  "
                                     + string(strerror(errno)), true);
                         schedfifo = false;
@@ -783,20 +842,20 @@ bool Config::startThread(pthread_t *pth, void *(*thread_fn)(void*), void *arg,
                 break;
             }
             else
-                Log("Failed to set thread detach state: " + asString(chk), true);
+                Log("Failed to set thread detach state " + asString(chk), true);
             pthread_attr_destroy(&attr);
         }
         else
-            Log("Failed to initialise thread attributes: " + asString(chk), true);
+            Log("Failed to initialise thread attributes " + asString(chk), true);
 
         if (schedfifo)
         {
-            Log("Failed to start thread (sched_fifo): " + asString(chk)
+            Log("Failed to start thread (sched_fifo) " + asString(chk)
                 + "  " + string(strerror(errno)), true);
             schedfifo = false;
             continue;
         }
-        Log("Failed to start thread (sched_other): " + asString(chk)
+        Log("Failed to start thread (sched_other) " + asString(chk)
             + "  " + string(strerror(errno)), true);
         outcome = false;
         break;
@@ -968,25 +1027,48 @@ LinuxSampler src/common/Features.cpp, licensed thus -
 static error_t parse_cmds (int key, char *arg, struct argp_state *state)
 {
     Config *settings = (Config*)state->input;
+    if (arg && arg[0] == 0x3d)
+        ++ arg;
+    int num;
+
     switch (key)
     {
         case 'c': settings->showConsole = true; break;
         case 'N': settings->nameTag = string(arg); break;
         case 'l': settings->paramsLoad = string(arg); break;
         case 'L': settings->instrumentLoad = string(arg); break;
-        case 'R': settings->Samplerate = Config::string2int(string(arg)); break;
-        case 'b': settings->Buffersize = Config::string2int(string(arg)); break;
-        case 'o': settings->Oscilsize = Config::string2int(string(arg)); break;
         case 'A':
             settings->audioEngine = alsa_audio;
             if (arg)
                 settings->audioDevice = string(arg);
+            else
+                settings->audioDevice = settings->alsaAudioDevice;
             break;
         case 'a':
             settings->midiEngine = alsa_midi;
             if (arg)
                 settings->midiDevice = string(arg);
             break;
+        case 'b': // messy but I can't think of a better way :(
+            num = Config::string2int(string(arg));
+            if (num >= 1024)
+                num = 1024;
+            else if (num >= 512)
+                num = 512;
+            else if (num >= 256)
+                num = 256;
+            else if (num >= 128)
+                num = 128;
+            else if (num >= 64)
+                num = 64;
+            else
+                num = 32;
+            settings->Buffersize = num;
+            break;
+        case 'D':
+            if (arg)
+                settings->rootDefine = string(arg);
+            break;
         case 'i':
             settings->showGui = false;
             settings->showConsole = false;
@@ -1003,6 +1085,35 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
             break;
         case 'k': settings->startJack = true; break;
         case 'K': settings->connectJackaudio = true; break;
+        case 'o':
+            num = Config::string2int(string(arg));
+            if (num >= 16384)
+                num = 16384;
+            else if (num >= 8192)
+                num = 8192;
+            else if (num >= 4096)
+                num = 4096;
+            else if (num >= 2048)
+                num = 2048;
+            else if (num >= 1024)
+                num = 1024;
+            else if (num >= 512)
+                num = 512;
+            else if (num >= 256)
+                num = 256;
+            else num = 128;
+            settings->Oscilsize = num;
+            break;
+        case 'R':
+            num = Config::string2int(string(arg));
+            if (num >= 96000)
+                num = 96000;
+            else if (num >= 48000)
+                num = 48000;
+            else
+                num = 44100;
+            settings->Samplerate = num;
+            break;
         case 'S':
             settings->restoreState = true;
             if (arg)
@@ -1021,7 +1132,8 @@ static error_t parse_cmds (int key, char *arg, struct argp_state *state)
         case ARGP_KEY_ARG:
         case ARGP_KEY_END:
             break;
-        default: return ARGP_ERR_UNKNOWN;
+        default:
+            return ARGP_ERR_UNKNOWN;
     }
     return 0;
 }
@@ -1036,3 +1148,102 @@ void Config::loadCmdArgs(int argc, char **argv)
     if (jackSessionUuid.size() && jackSessionFile.size())
         restoreJackSession = true;
 }
+
+void GuiThreadMsg::processGuiMessages()
+{
+    GuiThreadMsg *msg = (GuiThreadMsg *)Fl::thread_message();
+    if(msg)
+    {
+        switch(msg->type)
+        {
+        case GuiThreadMsg::NewSynthEngine:
+        {
+            SynthEngine *synth = ((SynthEngine *)msg->data);
+            MasterUI *guiMaster = synth->getGuiMaster();
+            if(!guiMaster)
+            {
+                cerr << "Error starting Main UI!" << endl;
+            }
+            else
+            {
+                guiMaster->Init(guiMaster->getSynth()->getWindowTitle().c_str());
+            }
+        }
+            break;
+        case GuiThreadMsg::UpdateMaster:
+        {
+            SynthEngine *synth = ((SynthEngine *)msg->data);
+            MasterUI *guiMaster = synth->getGuiMaster(false);
+            if(guiMaster)
+            {
+                guiMaster->refresh_master_ui();
+            }
+        }
+            break;
+        case GuiThreadMsg::UpdateConfig:
+        {
+            SynthEngine *synth = ((SynthEngine *)msg->data);
+            MasterUI *guiMaster = synth->getGuiMaster(false);
+            if(guiMaster)
+            {
+                guiMaster->configui->update_config(msg->index);
+            }
+        }
+            break;
+        case GuiThreadMsg::UpdatePanel:
+        {
+            SynthEngine *synth = ((SynthEngine *)msg->data);
+            MasterUI *guiMaster = synth->getGuiMaster(false);
+            if(guiMaster)
+            {
+                guiMaster->updatepanel();
+            }
+        }
+            break;
+        case GuiThreadMsg::UpdatePanelItem:
+            if(msg->index < NUM_MIDI_PARTS && msg->data)
+            {
+                SynthEngine *synth = ((SynthEngine *)msg->data);
+                MasterUI *guiMaster = synth->getGuiMaster(false);
+                if(guiMaster)
+                {
+                    guiMaster->panellistitem[(msg->index) % NUM_MIDI_CHANNELS]->refresh();
+                    guiMaster->updatepart();
+                }
+            }
+            break;
+        case GuiThreadMsg::UpdatePartProgram:
+            if(msg->index < NUM_MIDI_PARTS && msg->data)
+            {
+                SynthEngine *synth = ((SynthEngine *)msg->data);
+                MasterUI *guiMaster = synth->getGuiMaster(false);
+                if(guiMaster)
+                {
+                    guiMaster->updatepartprogram(msg->index);
+                }
+            }
+            break;
+        case GuiThreadMsg::UpdateEffects:
+            if(msg->data)
+            {
+                SynthEngine *synth = ((SynthEngine *)msg->data);
+                MasterUI *guiMaster = synth->getGuiMaster(false);
+                if(guiMaster)
+                {
+                    guiMaster->updateeffects(msg->index);
+                }
+            }
+            break;
+        case GuiThreadMsg::RegisterAudioPort:
+            if(msg->data)
+            {
+                SynthEngine *synth = ((SynthEngine *)msg->data);
+                mainRegisterAudioPort(synth, msg->index);
+            }
+            break;
+        default:
+            break;
+        }
+        delete msg;
+    }
+}
