@@ -36,6 +36,7 @@ using namespace std;
 #include "MusicIO/MusicClient.h"
 #include "Misc/HistoryListItem.h"
 #include "Misc/MiscFuncs.h"
+#include "FL/Fl.H"
 
 typedef enum { no_audio = 0, jack_audio, alsa_audio, } audio_drivers;
 typedef enum { no_midi = 0, jack_midi, alsa_midi, } midi_drivers;
@@ -54,16 +55,13 @@ class Config : public MiscFuncs
         void StartupReport(MusicClient *musicClient);
         void Announce(void);
         void Usage(void);
-    #if defined(CONSOLE_ERRORS)
         void Log(string msg, bool tostderr = false);
-    #else
-        void Log(string msg, bool tostderr = true);
-    #endif
         void flushLog(void);
 
         void clearPresetsDirlist(void);
 
         string testCCvalue(int cc);
+        string masterCCtest(int cc);
         void saveConfig(void);
         void saveState() { saveSessionData(StateFile); }
         void saveState(const string statefile)  { saveSessionData(statefile); }
@@ -92,6 +90,7 @@ class Config : public MiscFuncs
         string        ConfigFile;
         string        paramsLoad;
         string        instrumentLoad;
+        string        rootDefine;
         bool          restoreState;
         string        StateFile;
         string        CurrentXMZ;
@@ -122,20 +121,44 @@ class Config : public MiscFuncs
         string        alsaMidiDevice;
         string        nameTag;
 
-        int           BankUIAutoClose;
         unsigned int  GzipCompression;
         int           Interpolation;        
-        string        presetsDirlist[MAX_BANK_ROOT_DIRS];
-        int           CheckPADsynth;
+        string        presetsDirlist[MAX_PRESETS];
+        int           checksynthengines;
         bool          SimpleCheck;
         int           EnableProgChange;
+        bool          consoleMenuItem;
         int           rtprio;
         int           midi_bank_root;
         int           midi_bank_C;
         int           midi_upper_voice_C;
         int           enable_part_on_voice_load;
         int           single_row_panel;
-
+        int           NumAvailableParts;
+        unsigned char nrpnL;
+        unsigned char nrpnH;
+        unsigned char dataL;
+        unsigned char dataH;
+        bool          nrpnActive;
+        
+        struct IOdata{
+            unsigned char vectorXaxis[NUM_MIDI_CHANNELS];
+            unsigned char vectorYaxis[NUM_MIDI_CHANNELS];
+            unsigned char vectorXfeatures[NUM_MIDI_CHANNELS];
+            unsigned char vectorYfeatures[NUM_MIDI_CHANNELS];
+            unsigned char vectorXcc2[NUM_MIDI_CHANNELS];
+            unsigned char vectorYcc2[NUM_MIDI_CHANNELS];
+            unsigned char vectorXcc4[NUM_MIDI_CHANNELS];
+            unsigned char vectorYcc4[NUM_MIDI_CHANNELS];
+            unsigned char vectorXcc8[NUM_MIDI_CHANNELS];
+            unsigned char vectorYcc8[NUM_MIDI_CHANNELS];
+            int Part;
+            int Controller;
+            bool vectorEnabled[NUM_MIDI_CHANNELS];
+        };
+        
+        IOdata nrpndata;
+        
         deque<HistoryListItem> ParamsHistory;
         deque<HistoryListItem>::iterator itx;
         static const unsigned short MaxParamsHistory;
@@ -169,4 +192,42 @@ class Config : public MiscFuncs
         friend class YoshimiLV2Plugin;
 };
 
+//struct GuiThreadMsg must be allocated by caller via `new` and is freed by receiver via `delete`
+class GuiThreadMsg
+{
+private:
+    GuiThreadMsg()
+    {
+        data = NULL;
+        length = 0;
+        type = GuiThreadMsg::UNDEFINED;
+    }
+public:
+    enum
+    {
+        NewSynthEngine = 0,
+        UpdateMaster,
+        UpdateConfig,
+        UpdatePanel,
+        UpdatePanelItem,
+        UpdatePartProgram,
+        UpdateEffects,
+        RegisterAudioPort,
+        UNDEFINED = 9999
+    };
+    void *data; //custom data, must be static or handled by called, does nod freed by receiver
+    unsigned long length; //length of data member (determined by type member, can be set to 0, if data is known struct/class)
+    unsigned int index; // if there is integer data, it can be passed through index (to remove aditional receiver logic)
+    unsigned int type; // type of gui message (see enum above)
+    static void sendMessage(void *_data, unsigned int _type, unsigned int _index)
+    {
+        GuiThreadMsg *msg = new GuiThreadMsg;
+        msg->data = _data;
+        msg->type = _type;
+        msg->index = _index;
+        Fl::awake((void *)msg);
+    }
+    static void processGuiMessages();
+};
+
 #endif
