@@ -6,7 +6,7 @@
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
     Public License as published by the Free Software Foundation, either
-    version 3 of the License, or (at your option) any later version.
+    version 2 of the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +19,7 @@
 
 #include <sys/stat.h>
 #include <sstream>
+#include <string.h>
 
 using namespace std;
 
@@ -116,14 +117,16 @@ string MiscFuncs::asString(long long n)
    oss << n;
    return string(oss.str());
 }
-#ifdef __x86_64__
-string MiscFuncs::asString(size_t n)
+
+//#if !defined( __arm__ ) && !defined( __i386__ )
+//string MiscFuncs::asString(size_t n)
+string MiscFuncs::asString(unsigned long n)
 {
     ostringstream oss;
     oss << n;
     return string(oss.str());
 }
-#endif
+//#endif
 
 string MiscFuncs::asString(long n)
 {
@@ -147,6 +150,14 @@ string MiscFuncs::asString(unsigned int n, unsigned int width)
 }
 
 
+string MiscFuncs::asString(unsigned char c)
+{
+    ostringstream oss;
+    oss.width(1);
+    oss << c;
+    return oss.str();
+}
+
 string MiscFuncs::asString(float n)
 {
    ostringstream oss;
@@ -179,3 +190,36 @@ string MiscFuncs::asHexString(unsigned int x)
    oss << hex << x;
    return string(oss.str());
 }
+
+
+char *MiscFuncs::skipSpace(char *buf)
+{
+    while (buf[0] == 0x20)
+    {
+        ++ buf;
+    }
+    return buf;
+}
+
+char *MiscFuncs::skipChars(char *buf)
+{
+    while (buf[0] > 0x20) // will also stop on line ends
+    {
+        ++ buf;
+    }
+    if (buf[0] == 0x20)
+        buf = skipSpace(buf);
+    return buf;
+}
+
+
+int MiscFuncs::matchWord(char *buf, const char *word)
+{
+    int newp = 0;
+    int size = strlen(word);
+    while (buf[newp] > 0x20 && buf[newp] < 0x7f && newp < size && (buf[newp] | 0x20) == word[newp])
+            ++ newp;
+    if (newp == size)
+        return size;
+    return 0;
+}
