@@ -57,7 +57,7 @@ class SynthEngine : private SynthHelper, MiscFuncs
         bool isLV2Plugin;
         Bank bank;
         Config Runtime;
-        PresetsStore presetsstore;
+        PresetsStore presetsstore;        
     public:
         SynthEngine(int argc, char **argv, bool _isLV2Plugin = false, unsigned int forceId = 0);
         ~SynthEngine();
@@ -80,13 +80,24 @@ class SynthEngine : private SynthHelper, MiscFuncs
         void NoteOn(unsigned char chan, unsigned char note, unsigned char velocity);
         void NoteOff(unsigned char chan, unsigned char note);
         void SetController(unsigned char chan, int type, short int par);
+        void SetZynControls();
         void SetBankRoot(int rootnum);
         void SetBank(int banknum);
-        void SetProgram(unsigned char chan, unsigned char pgm);
+        void SetProgram(unsigned char chan, unsigned short pgm);
+        void SetPartChan(unsigned char npart, unsigned char nchan);
+        void SetPartDestination(unsigned char npart, unsigned char dest);
+        void SetSystemValue(int type, int value);
+        void DecodeCommands(char *buffer);
+        int commandSet(char *point);
+        int commandVector(char *point);
+        bool vectorInit(int dHigh, unsigned char chan, int par);
+        void vectorSet(int dHigh, unsigned char chan, int par);
+        void ClearNRPNs(void);
         float numRandom(void);
         unsigned int random(void);
         void ShutUp(void);
-        void MasterAudio(float *outl [NUM_MIDI_PARTS], float *outr [NUM_MIDI_PARTS], int to_process = 0);
+        void allStop();
+        int MasterAudio(float *outl [NUM_MIDI_PARTS + 1], float *outr [NUM_MIDI_PARTS + 1], int to_process = 0);
         void partonoff(int npart, int what);
         void Mute(void) { __sync_or_and_fetch(&muted, 0xFF); }
         void Unmute(void) { __sync_and_and_fetch(&muted, 0); }
@@ -94,23 +105,25 @@ class SynthEngine : private SynthHelper, MiscFuncs
 
         Part *part[NUM_MIDI_PARTS];
         bool shutup;
+        float fadeStep;
+        float fadeLevel;
 
         // parameters
         unsigned int samplerate;
         float samplerate_f;
         float halfsamplerate_f;
         int buffersize;
-        float buffersize_f;
+        float buffersize_f;        
         int bufferbytes;
         int oscilsize;
         float oscilsize_f;
         int halfoscilsize;
         float halfoscilsize_f;
 
-        int processOffset; //used for variable length runs
         int p_buffersize; //used for variable length runs
         int p_bufferbytes; //used for variable length runs
         float p_buffersize_f; //used for variable length runs
+        float p_all_buffersize_f; //used for variable length runs (mainly for lv2 - calculate envelopes and lfo)
 
         unsigned char Pvolume;
         int           Paudiodest;
@@ -202,6 +215,7 @@ class SynthEngine : private SynthHelper, MiscFuncs
 
         int LFOtime; // used by Pcontinous
         string windowTitle;
+        MusicClient *musicClient;
 };
 
 inline float SynthEngine::numRandom(void)
