@@ -53,6 +53,8 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth) :
     synth(_synth)
 {
     information.PADsynth_used = 0;
+    information.ADDsynth_used = 0;
+    information.SUBsynth_used = 0;
     memset(&parentstack, 0, sizeof(parentstack));
     tree = mxmlNewElement(MXML_NO_PARENT, "?xml version=\"1.0\" encoding=\"UTF-8\"?");
     mxml_node_t *doctype = mxmlNewElement(tree, "!DOCTYPE");
@@ -62,11 +64,18 @@ XMLwrapper::XMLwrapper(SynthEngine *_synth) :
 
     mxmlElementSetAttr(root, "version-major", "1");
     mxmlElementSetAttr(root, "version-minor", "1");
-//    mxmlElementSetAttr(root, "ZynAddSubFX-author", "Alan Calvert");
+    mxmlElementSetAttr(root, "ZynAddSubFX-author", "Nasca Octavian Paul");
+
+    mxmlElementSetAttr(root, "Yoshimi-author", "Alan Ernest Calvert");
+    string tmp = YOSHIMI_VERSION;
+    string::size_type pos1 = tmp.find('.'); // != string::npos
+    string::size_type pos2 = tmp.find('.',pos1+1);
+    mxmlElementSetAttr(root, "Yoshimi-major", tmp.substr(0, pos1).c_str());
+    mxmlElementSetAttr(root, "Yoshimi-minor", tmp.substr(pos1+1, pos2-pos1-1).c_str());
 
     info = addparams0("INFORMATION"); // specifications
     beginbranch("BASE_PARAMETERS");
-    addpar("max_midi_parts", NUM_MIDI_PARTS);
+    addpar("max_midi_parts", synth->getRuntime().NumAvailableParts);
     addpar("max_kit_items_per_instrument", NUM_KIT_ITEMS);
 
     addpar("max_system_effects", NUM_SYS_EFX);
@@ -96,53 +105,136 @@ bool XMLwrapper::checkfileinformation(const string& filename)
     char *xmldata = doloadfile(filename);
     if (!xmldata)
         return -1;
+    
+    bool bRet = false; // we're not actually using this!
     char *start = strstr(xmldata, "<INFORMATION>");
     char *end = strstr(xmldata, "</INFORMATION>");
-    if (!start || !end || start > end)
+    if (!start || !end || start >= end)
     {
+        bRet = slowinfosearch(xmldata);
         delete [] xmldata;
-        return false;
+        return bRet;
     }
-    //Andrew: just make it simple
-    bool bRet = false;
-    if(strstr(start, "name=\"PADsynth_used\" value=\"yes\""))
+
+    // Andrew: just make it simple
+    // Will: but not too simple :)
+    char *idx = start;
+    *end = 0; // fiddle to limit search - I can't find a better way :(
+    unsigned short names = 0;
+    
+    /* the following could be in any order. We are checking for
+     * the actual exisitence of the fields as well as their value.
+     */
+    idx = strstr(start, "name=\"ADDsynth_used\"");
+    if (idx != NULL)
+    {
+        names |= 2;
+        if(strstr(idx, "name=\"ADDsynth_used\" value=\"yes\""))
+            information.ADDsynth_used = 1;
+    }
+    
+    idx = strstr(start, "name=\"SUBsynth_used\"");
+    if (idx != NULL)
+    {
+        names |= 4;
+        if(strstr(idx, "name=\"SUBsynth_used\" value=\"yes\""))
+            information.SUBsynth_used = 1;
+    }
+    
+    idx = strstr(start, "name=\"PADsynth_used\"");
+    if (idx != NULL)
+    {
+        names |= 1;
+        if(strstr(idx, "name=\"PADsynth_used\" value=\"yes\""))
+            information.PADsynth_used = 1;
+    }
+    
+    if (names == 7)
     {
-        information.PADsynth_used = 1;
         bRet = true;
+//        if (information.SUBsynth_used)
+//                synth->getRuntime().Log("Sub found");
+    }
+    else
+    {
+//        if (strstr(idx, "<INSTRUMENT_KIT>"))
+//            synth->getRuntime().Log("Oops! Shouldn't find it.");
+        *end = 0x3c; // restore full length
+        bRet = slowinfosearch(xmldata);
     }
     delete [] xmldata;
     return bRet;
+}
 
 
+bool XMLwrapper::slowinfosearch(char *idx)
+{
+    idx = strstr(idx, "<INSTRUMENT_KIT>");
+    if (idx == NULL)
+        return false;
+    
+    string mark;
+    int max = NUM_KIT_ITEMS;
+    
     /*
-    end += strlen("</INFORMATION>");
-    //end[0] = '\0';
-    //tree = mxmlNewElement(MXML_NO_PARENT, "?xml version=\"1.0\" encoding=\"UTF-8\"?");
-    tree = node = root = mxmlLoadString(NULL, xmldata, MXML_OPAQUE_CALLBACK);
-    if (!root)
-    {
-        delete [] xmldata;
-        mxmlDelete(tree);
-        node = root = tree = NULL;
+     * The following *must* exist, otherwise the file is corrupted.
+     * They will always be in this order, which means we only need
+     * to scan once through the file.
+     * We can stop if we get to a point where ADD, SUB and PAD
+     * have all been enabled.
+     */
+    idx = strstr(idx, "name=\"kit_mode\"");
+    if (idx == NULL)
         return false;
-    }
-    root = mxmlFindElement(tree, tree, "INFORMATION", NULL, NULL, MXML_DESCEND);
-    push(root);
-    if (!root)
+    if (strncmp(idx + 16 , "value=\"0\"", 9) == 0)
+        max = 1;
+
+    for (int kitnum = 0; kitnum < max; ++kitnum)
     {
-        delete [] xmldata;
-        mxmlDelete(tree);
-        node = root = tree = NULL;
-        return false;
+        mark = "<INSTRUMENT_KIT_ITEM id=\"" + asString(kitnum) + "\">";
+        idx = strstr(idx, mark.c_str());
+        if (idx == NULL)
+            return false;
+        
+        idx = strstr(idx, "name=\"enabled\"");
+        if (idx == NULL)
+            return false;
+        if (!strstr(idx, "name=\"enabled\" value=\"yes\""))
+            continue;
+        
+        if (!information.ADDsynth_used)
+        {
+            idx = strstr(idx, "name=\"add_enabled\"");
+            if (idx == NULL)
+                return false;
+            if (strncmp(idx + 26 , "yes", 3) == 0)
+                information.ADDsynth_used = 1;
+        }
+        if (!information.SUBsynth_used)
+        {
+            idx = strstr(idx, "name=\"sub_enabled\"");
+            if (idx == NULL)
+                return false;
+            if (strncmp(idx + 26 , "yes", 3) == 0)
+                information.SUBsynth_used = 1;
+        }
+        if (!information.PADsynth_used)
+        {
+            idx = strstr(idx, "name=\"pad_enabled\"");
+            if (idx == NULL)
+                return false;
+            if (strncmp(idx + 26 , "yes", 3) == 0)
+                information.PADsynth_used = 1;
+        }
+        if (information.ADDsynth_used
+          & information.SUBsynth_used
+          & information.PADsynth_used)
+        {
+//            synth->getRuntime().Log("Kit count " + asString(kitnum));
+            break;
+        }
     }
-    information.PADsynth_used = getparbool("PADsynth_used",0);
-    exitbranch();
-    if (tree)
-        mxmlDelete(tree);
-    delete [] xmldata;
-    node = root = tree = NULL;
-    return true;
-    */
+  return true;
 }
 
 
@@ -153,7 +245,7 @@ bool XMLwrapper::saveXMLfile(const string& filename)
     char *xmldata = getXMLdata();
     if (!xmldata)
     {
-        synth->getRuntime().Log("Error, failed to allocate xml data space");
+        synth->getRuntime().Log("XML: Failed to allocate xml data space");
         return false;
     }
     unsigned int compression = synth->getRuntime().GzipCompression;
@@ -162,7 +254,7 @@ bool XMLwrapper::saveXMLfile(const string& filename)
         FILE *xmlfile = fopen(filename.c_str(), "w");
         if (!xmlfile)
         {
-            synth->getRuntime().Log("Error, failed to open xml file " + filename + " for save");
+            synth->getRuntime().Log("XML: Failed to open xml file " + filename + " for save");
             return false;
         }
         fputs(xmldata, xmlfile);
@@ -179,7 +271,7 @@ bool XMLwrapper::saveXMLfile(const string& filename)
         gzfile = gzopen(filename.c_str(), options);
         if (gzfile == NULL)
         {
-            synth->getRuntime().Log("Error, gzopen() == NULL");
+            synth->getRuntime().Log("XML: gzopen() == NULL");
             return false;
         }
         gzputs(gzfile, xmldata);
@@ -196,6 +288,8 @@ char *XMLwrapper::getXMLdata()
     memset(tabs, 0, STACKSIZE + 2);
     mxml_node_t *oldnode=node;
     node = info;
+    addparbool("ADDsynth_used", information.ADDsynth_used);
+    addparbool("SUBsynth_used", information.SUBsynth_used);
     addparbool("PADsynth_used", information.PADsynth_used);
     node = oldnode;
     char *xmldata = mxmlSaveAllocString(tree, XMLwrapper_whitespace_callback);
@@ -264,25 +358,35 @@ bool XMLwrapper::loadXMLfile(const string& filename)
     const char *xmldata = doloadfile(filename);
     if (xmldata == NULL)
     {
-        synth->getRuntime().Log("Could not load xml file: " + filename);
+        synth->getRuntime().Log("XML: Could not load xml file: " + filename);
          return false;
     }
     root = tree = mxmlLoadString(NULL, xmldata, MXML_OPAQUE_CALLBACK);
     delete [] xmldata;
     if (!tree)
     {
-        synth->getRuntime().Log("File " + filename + " is not XML");
+        synth->getRuntime().Log("XML: File " + filename + " is not XML");
         return false;
     }
     node = root = mxmlFindElement(tree, tree, "ZynAddSubFX-data", NULL, NULL, MXML_DESCEND);
     if (!root)
     {
-        synth->getRuntime().Log("File " + filename + " doesn't contain valid data in this context");
+        synth->getRuntime().Log("XML: File " + filename + " doesn't contain valid data in this context");
         return false;
     }
     push(root);
     xml_version.major = string2int(mxmlElementGetAttr(root, "version-major"));
     xml_version.minor = string2int(mxmlElementGetAttr(root, "version-minor"));
+    if (mxmlElementGetAttr(root, "Yoshimi-major"))
+    {
+        xml_version.y_major = string2int(mxmlElementGetAttr(root, "Yoshimi-major"));
+//        synth->getRuntime().Log("XML: Yoshimi " + asString(xml_version.y_major));
+    }
+    if (mxmlElementGetAttr(root, "Yoshimi-minor"))
+    {
+        xml_version.y_minor = string2int(mxmlElementGetAttr(root, "Yoshimi-minor"));
+//        synth->getRuntime().Log("XML: Yoshimi " + asString(xml_version.y_minor));
+    }
     return true;
 }
 
@@ -293,7 +397,7 @@ char *XMLwrapper::doloadfile(const string& filename)
     gzFile gzf  = gzopen(filename.c_str(), "rb");
     if (!gzf)
     {
-        synth->getRuntime().Log("Failed to open xml file " + filename + " for load, errno: "
+        synth->getRuntime().Log("XML: Failed to open xml file " + filename + " for load, errno: "
                     + asString(errno) + "  " + string(strerror(errno)));
         return NULL;
     }
@@ -314,9 +418,9 @@ char *XMLwrapper::doloadfile(const string& filename)
         else if (this_read < 0)
         {
             int errnum;
-            synth->getRuntime().Log("Read error in zlib: " + string(gzerror(gzf, &errnum)));
+            synth->getRuntime().Log("XML: Read error in zlib: " + string(gzerror(gzf, &errnum)));
             if (errnum == Z_ERRNO)
-                synth->getRuntime().Log("Filesystem error: " + string(strerror(errno)));
+                synth->getRuntime().Log("XML: Filesystem error: " + string(strerror(errno)));
             quit = true;
         }
         else if (total_bytes > 0)
@@ -497,7 +601,7 @@ void XMLwrapper::push(mxml_node_t *node)
 {
     if (stackpos >= STACKSIZE - 1)
     {
-        synth->getRuntime().Log("Not good, XMLwrapper push on a full parentstack");
+        synth->getRuntime().Log("XML: Not good, XMLwrapper push on a full parentstack");
         return;
     }
     stackpos++;
@@ -509,7 +613,7 @@ mxml_node_t *XMLwrapper::pop(void)
 {
     if (stackpos <= 0)
     {
-        synth->getRuntime().Log("Not good, XMLwrapper pop on empty parentstack");
+        synth->getRuntime().Log("XML: Not good, XMLwrapper pop on empty parentstack");
         return root;
     }
     mxml_node_t *node = parentstack[stackpos];
@@ -523,7 +627,7 @@ mxml_node_t *XMLwrapper::peek(void)
 {
     if (stackpos <= 0)
     {
-        synth->getRuntime().Log("Not good, XMLwrapper peek on an empty parentstack");
+        synth->getRuntime().Log("XML: Not good, XMLwrapper peek on an empty parentstack");
         return root;
     }
     return parentstack[stackpos];
