@@ -7,7 +7,7 @@
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
     Public License as published by the Free Software Foundation, either
-    version 3 of the License, or (at your option) any later version.
+    version 2 of the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,7 +67,10 @@ bool JackEngine::connectServer(string server)
         return true;
     }
     else
+    {
         synth->getRuntime().Log("Failed to open jack client on server " + server);
+        splashMessages.push_back("Can't connect to jack audio :(");
+    }
     return false;
 }
 
@@ -134,6 +137,7 @@ bool JackEngine::openJackClient(string server)
 bool JackEngine::Start(void)
 {
     bool jackPortsRegistered = true;
+    internalbuff = synth->getRuntime().Buffersize;
     //Andrew Deryabin: use default error callback function provided by jack
     //jack_set_error_function(_errorCallback);
     jack_set_xrun_callback(jackClient, _xrunCallback, this);
@@ -161,6 +165,7 @@ bool JackEngine::Start(void)
         goto bail_out;
     }
 
+    /*
     for (int port = 0; port < (2 * NUM_MIDI_PARTS + 2); ++port) // include mains
     {
         if (!audio.ports[port])
@@ -169,6 +174,7 @@ bool JackEngine::Start(void)
             break;
         }
     }
+    */
 
     if (!jack_activate(jackClient) && jackPortsRegistered)
     {
@@ -206,10 +212,9 @@ void JackEngine::Close(void)
         synth->getRuntime().runSynth = false;
     }
 
-    if (sem_post(&midiSem) == 0)
+    if(midi.pThread != 0) //wait for midi thread to finish
     {
-
-        if(midi.pThread != 0) //wait for midi thread to finish
+        if (sem_post(&midiSem) == 0)
         {
             void *ret = NULL;
             pthread_join(midi.pThread, &ret);
@@ -244,32 +249,64 @@ void JackEngine::Close(void)
     }
 }
 
+void JackEngine::registerJackPort(int partnum)
+{
+    int portnum = partnum * 2;
+    if(partnum >=0 && partnum < NUM_MIDI_PARTS)
+    {
+        if(audio.ports [portnum] != NULL)
+        {
+            //port already registered
+            //synth->getRuntime().Log("Jack port " + asString(partnum) + " already registered!");
+            return;
+        }
+        if(synth->part [partnum] && synth->part [partnum]->Penabled)
+        {
+            string portName = "track_" + asString(partnum + 1) + "_r";
+            audio.ports[portnum + 1] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+            portName = "track_" + asString(partnum + 1) + "_l";
+            audio.ports[portnum] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+
+            if(audio.ports [portnum])
+            {
+                synth->getRuntime().Log("Registered jack port " + asString(partnum));
+            }
+            else
+            {
+                synth->getRuntime().Log("Error registering jack port " + asString(partnum));
+            }
+        }
+    }
+}
 
 bool JackEngine::openAudio(void)
 {
     // Register mixed outputs
     audio.ports[2 * NUM_MIDI_PARTS] = jack_port_register(jackClient, "left", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
     audio.ports[2 * NUM_MIDI_PARTS + 1] = jack_port_register(jackClient, "right", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-    // And individual parts
-    for (int port = 0; port < 2 * NUM_MIDI_PARTS; ++port)
-    {
-        stringstream portName;
-        portName << "track_" << ((port / 2) + 1) << ((port % 2) ? "_r" : "_l");
-        audio.ports[port] = jack_port_register(jackClient, portName.str().c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-    }
 
-    bool jackPortsRegistered = true;
-    for (int port = 0; port < (2 * NUM_MIDI_PARTS + 2); ++port)
+    // And individual parts
+/* this section is now reduntant!
+    for (int port = 0; port < NUM_MIDI_PARTS; ++port)
     {
+        //stringstream portName;
+        //portName << "track_" << ((port / 2) + 1) << ((port % 2) ? "_r" : "_l");
+        //audio.ports[port] = jack_port_register(jackClient, portName.str().c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+        registerJackPort(port);
+    }
+*/
+    bool jackPortsRegistered = true;
+    /*for (int port = 0; port < (2 * NUM_MIDI_PARTS + 2); ++port)
+    {        
         if (!audio.ports[port])
         {
             jackPortsRegistered = false;
             break;
         }
-    }
+    }*/
 
     if (jackPortsRegistered)
-        return prepBuffers(false) && latencyPrep();
+        return prepBuffers() && latencyPrep();
     else
         synth->getRuntime().Log("Failed to register jack audio ports");
     Close();
@@ -313,7 +350,7 @@ bool JackEngine::connectJackPorts(void)
         return false;
 	}
     int ret;
-    for (int port = 0; port < 2 && (NULL != audio.ports[port]); ++port)
+    for (int port = 0; port < 2 && (NULL != audio.ports[port + NUM_MIDI_PARTS*2]); ++port)
     {
         const char *port_name = jack_port_name(audio.ports[port + NUM_MIDI_PARTS * 2]);
         if ((ret = jack_connect(jackClient, port_name, playback_ports[port])))
@@ -327,7 +364,6 @@ bool JackEngine::connectJackPorts(void)
     return true;
 }
 
-
 int JackEngine::clientId(void)
 {
     if (jackClient)
@@ -368,39 +404,92 @@ int JackEngine::processCallback(jack_nframes_t nframes)
 
 bool JackEngine::processAudio(jack_nframes_t nframes)
 {
-    for (int port = 0; port < (2*NUM_MIDI_PARTS+2); ++port)
+    // Part buffers
+    for (int port = 0; port < 2 * NUM_MIDI_PARTS; ++port)
     {
-        audio.portBuffs[port] =
-            (float*)jack_port_get_buffer(audio.ports[port], nframes);
-        if (!audio.portBuffs[port])
+        if(audio.ports [port])
         {
-            synth->getRuntime().Log("Failed to get jack audio port buffer: " + asString(port));
-            return false;
-        }
-    }
-    getAudio();
-    int framesize = sizeof(float) * nframes;
-    // Part outputs
-    for (int port, idx = 0; port < NUM_MIDI_PARTS; port++ , idx += 2)
-    {
-        if (synth->part[port]->Paudiodest & 2)
-        {
-            memcpy(audio.portBuffs[idx], zynLeft[port], framesize);
-            memcpy(audio.portBuffs[idx + 1], zynRight[port], framesize);
-        }
-        else
-        {
-            memset(audio.portBuffs[idx], 0, framesize);
-            memset(audio.portBuffs[idx + 1], 0, framesize);
+            audio.portBuffs[port] =
+                    (float*)jack_port_get_buffer(audio.ports[port], nframes);
+            if (!audio.portBuffs[port])
+            {
+                synth->getRuntime().Log("Failed to get jack audio port buffer: " + asString(port));
+                return false;
+            }
         }
     }
     // And mixed outputs
-    memcpy(audio.portBuffs[2 * NUM_MIDI_PARTS], zynLeft[NUM_MIDI_PARTS], framesize);
-    memcpy(audio.portBuffs[2 * NUM_MIDI_PARTS + 1], zynRight[NUM_MIDI_PARTS], framesize);
+    audio.portBuffs[2 * NUM_MIDI_PARTS] = (float*)jack_port_get_buffer(audio.ports[2 * NUM_MIDI_PARTS], nframes);
+    if (!audio.portBuffs[2 * NUM_MIDI_PARTS])
+    {
+        synth->getRuntime().Log("Failed to get jack audio port buffer: " + asString(2 * NUM_MIDI_PARTS));
+        return false;
+    }
+    audio.portBuffs[2 * NUM_MIDI_PARTS + 1] = (float*)jack_port_get_buffer(audio.ports[2 * NUM_MIDI_PARTS + 1], nframes);
+    if (!audio.portBuffs[2 * NUM_MIDI_PARTS + 1])
+    {
+        synth->getRuntime().Log("Failed to get jack audio port buffer: " + asString(2 * NUM_MIDI_PARTS + 1));
+        return false;
+    }    
+    //synth->getRuntime().Log("Jack " + asString(nframes) + "  Internal " + asString(internalbuff));
+    int framesize = sizeof(float) * nframes;
+    if (nframes < internalbuff)
+    {
+        synth->MasterAudio(zynLeft, zynRight, nframes);
+        sendAudio(framesize, 0);
+    }
+    else
+    {
+        for (unsigned int pos = 0; pos < nframes; pos += internalbuff)
+        {
+        synth->MasterAudio(zynLeft, zynRight, 0);
+        sendAudio(framesize, pos);
+        }
+    }
     return true;
 }
 
 
+void JackEngine::sendAudio(int framesize, unsigned int offset)
+{
+    // Part outputs
+    int currentmax = synth->getRuntime().NumAvailableParts;
+    for (int port = 0, idx = 0; idx < 2 * NUM_MIDI_PARTS; port++ , idx += 2)
+    {
+        if(audio.ports [idx])
+        {
+            if (jack_port_connected(audio.ports[idx])) // just a few % improvement.
+            {
+                float *lpoint = audio.portBuffs[idx] + offset;
+                float *rpoint = audio.portBuffs[idx + 1] + offset;
+                if ((synth->part[port]->Paudiodest & 2) && port < currentmax)
+                {
+                    
+//                    memcpy(audio.portBuffs[idx], zynLeft[port], framesize);
+//                    memcpy(audio.portBuffs[idx + 1], zynRight[port], framesize);
+                    memcpy(lpoint, zynLeft[port], framesize);
+                    memcpy(rpoint, zynRight[port], framesize);
+                }
+                else
+                {
+//                    memset(audio.portBuffs[idx], 0, framesize);
+//                    memset(audio.portBuffs[idx + 1], 0, framesize);
+                    memset(lpoint, 0, framesize);
+                    memset(rpoint, 0, framesize);
+                }
+            }
+        }
+    }
+    // And mixed outputs
+    float *Lpoint = audio.portBuffs[2 * NUM_MIDI_PARTS] + offset;
+    float *Rpoint = audio.portBuffs[2 * NUM_MIDI_PARTS + 1] + offset;
+    memcpy(Lpoint, zynLeft[NUM_MIDI_PARTS], framesize);
+    memcpy(Rpoint, zynRight[NUM_MIDI_PARTS], framesize);
+//    memcpy(audio.portBuffs[2 * NUM_MIDI_PARTS], zynLeft[NUM_MIDI_PARTS], framesize);
+//    memcpy(audio.portBuffs[2 * NUM_MIDI_PARTS + 1], zynRight[NUM_MIDI_PARTS], framesize);
+}
+
+
 bool JackEngine::processMidi(jack_nframes_t nframes)
 {
     void *portBuf = jack_port_get_buffer(midi.port, nframes);
@@ -422,7 +511,7 @@ bool JackEngine::processMidi(jack_nframes_t nframes)
     {
         if(!jack_midi_event_get(&jEvent, portBuf, idx))
         {
-            if (jEvent.size < 1 || jEvent.size > sizeof(event.data))
+            if (jEvent.size < 1 || (jEvent.size > sizeof(event.data)))
                 continue; // no interest in zero sized or long events
             event.time = jEvent.time;
             memset(event.data, 0, sizeof(event.data));
