@@ -52,8 +52,53 @@ void Controller::defaults(void)
     modwheel.exponential = 0;
     fmamp.receive = 1;
     volume.receive = 1;
+    volume.data = 96;
+    volume.volume = 96.0f/127.0f;
     sustain.receive = 1;
+    portamentosetup();
+    resonancecenter.depth = 64;
+    resonancebandwidth.depth = 64;
 
+    initportamento(440.0f, 440.0f, false);
+    setportamento(0);
+}
+
+
+void Controller::resetall()
+{
+    setpitchwheelbendrange(200); // 2 halftones
+    setpitchwheel(0); // center
+    expression.receive = 1;
+    setexpression(127);
+    setPanDepth(64);
+    filtercutoff.depth = 64;
+    setfiltercutoff(64);
+    setfilterq(64);
+    bandwidth.depth = 64;
+    bandwidth.exponential = 0;
+    setbandwidth(64);
+    modwheel.depth = 80;
+    modwheel.exponential = 0;
+    setmodwheel(64);
+    fmamp.receive = 1;
+    setfmamp(127);
+    volume.receive = 1;
+    setvolume(96);
+    sustain.receive = 1;
+    setsustain(0);
+    setlegato(0);
+    portamentosetup();
+    initportamento(440.0f, 440.0f, false);
+    setportamento(0);
+    resonancecenter.depth = 64;
+    setresonancecenter(64);
+    resonancebandwidth.depth = 64;
+    setresonancebw(64);
+}
+
+
+void  Controller::portamentosetup()
+{
     portamento.portamento = 0;
     portamento.used = 0;
     portamento.proportional = 0;
@@ -65,29 +110,6 @@ void Controller::defaults(void)
     portamento.pitchthresh = 3;
     portamento.pitchthreshtype = 1;
     portamento.noteusing = -1;
-    resonancecenter.depth = 64;
-    resonancebandwidth.depth = 64;
-
-    initportamento(440.0f, 440.0f, false);
-    setportamento(0);
-}
-
-
-void Controller::resetall()
-{
-    setpitchwheel(0); // center
-    setexpression(127);
-    setPanDepth(64);
-    setfiltercutoff(64);
-    setfilterq(64);
-    setbandwidth(64);
-    setmodwheel(64);
-    setfmamp(127);
-    setvolume(127);
-    setsustain(0);
-    setlegato(0);
-    setresonancecenter(64);
-    setresonancebw(64);
 }
 
 
@@ -180,13 +202,12 @@ void Controller::setfmamp(int value)
 }
 
 
-void Controller::setvolume(int value)
+void Controller::setvolume(int value) // range is 64 to 127
 {
+    if (value < 64)
+        value = 96; // set invalid to default
     volume.data = value;
-    if (volume.receive)
-        volume.volume = powf(0.1f, (127 - value) / 127.0f * 2.0f);
-    else
-        volume.volume = 0.9f;
+    volume.volume = value / 127.0f;
 }
 
 
@@ -260,7 +281,7 @@ int Controller::initportamento(float oldfreq, float newfreq, bool in_progress)
         portamentotime *= powf(0.1f, (64.0f - portamento.updowntimestretch) / 64.0f);
     }
 
-    portamento.dx = synth->buffersize_f / (portamentotime * synth->samplerate_f);
+    portamento.dx = synth->p_all_buffersize_f / (portamentotime * synth->samplerate_f);
     portamento.origfreqrap = oldfreq / newfreq;
 
     float tmprap = (portamento.origfreqrap > 1.0f)
@@ -321,6 +342,7 @@ void Controller::add2XML(XMLwrapper *xml)
     xml->addparbool("mod_wheel_exponential",modwheel.exponential);
     xml->addparbool("fm_amp_receive",fmamp.receive);
     xml->addparbool("volume_receive",volume.receive);
+    xml->addpar("volume_range",volume.data);
     xml->addparbool("sustain_receive",sustain.receive);
 
     xml->addparbool("portamento_receive",portamento.receive);
@@ -351,6 +373,8 @@ void Controller::getfromXML(XMLwrapper *xml)
     modwheel.exponential=xml->getparbool("mod_wheel_exponential",modwheel.exponential);
     fmamp.receive=xml->getparbool("fm_amp_receive",fmamp.receive);
     volume.receive=xml->getparbool("volume_receive",volume.receive);
+    setvolume(xml->getpar127("volume_range",volume.data));
+
     sustain.receive=xml->getparbool("sustain_receive",sustain.receive);
 
     portamento.receive=xml->getparbool("portamento_receive",portamento.receive);
