@@ -36,7 +36,7 @@ using namespace std;
 #include "Synth/PADnote.h"
 
 PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
-    float velocity, int portamento_, int midinote, bool besilent) :
+    float velocity, int portamento_, int midinote, bool besilent, SynthEngine *_synth) :
     ready(false),
     finished_(false),
     pars(parameters),
@@ -44,14 +44,15 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
     released(false),
     nsample(0),
     portamento(portamento_),
-    ctl(ctl_)
+    ctl(ctl_),
+    synth(_synth)
 
 {
 //    tmpwave = (float*)fftwf_malloc(synth->bufferbytes);
 
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
-    Legato.fade.length = lrintf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
+    Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
     if (Legato.fade.length < 1)
         Legato.fade.length = 1; // (if something's fishy)
     Legato.fade.step = (1.0 / Legato.fade.length);
@@ -105,7 +106,7 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
     if (size == 0)
         size = 1;
 
-    poshi_l = lrintf(synth->numRandom() * (size - 1));
+    poshi_l = (int)truncf(synth->numRandom() * (size - 1));
     if (pars->PStereo != 0)
         poshi_r = (poshi_l + size / 2) % size;
     else
@@ -137,11 +138,11 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
     } else
         NoteGlobalPar.Punch.Enabled = 0;
 
-    NoteGlobalPar.FreqEnvelope = new Envelope(pars->FreqEnvelope, basefreq);
-    NoteGlobalPar.FreqLfo = new LFO(pars->FreqLfo, basefreq);
+    NoteGlobalPar.FreqEnvelope = new Envelope(pars->FreqEnvelope, basefreq, synth);
+    NoteGlobalPar.FreqLfo = new LFO(pars->FreqLfo, basefreq, synth);
 
-    NoteGlobalPar.AmpEnvelope = new Envelope(pars->AmpEnvelope, basefreq);
-    NoteGlobalPar.AmpLfo = new LFO(pars->AmpLfo, basefreq);
+    NoteGlobalPar.AmpEnvelope = new Envelope(pars->AmpEnvelope, basefreq, synth);
+    NoteGlobalPar.AmpLfo = new LFO(pars->AmpLfo, basefreq, synth);
 
     NoteGlobalPar.Volume =
         4.0f * powf(0.1f, 3.0f * (1.0f - pars->PVolume / 96.0f)) //-60 dB .. 0 dB
@@ -154,12 +155,12 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
         * NoteGlobalPar.AmpLfo->amplfoout();
 
     NoteGlobalPar.GlobalFilterL =
-        new Filter(pars->GlobalFilter);
+        new Filter(pars->GlobalFilter, synth);
     NoteGlobalPar.GlobalFilterR =
-        new Filter(pars->GlobalFilter);
+        new Filter(pars->GlobalFilter, synth);
 
-    NoteGlobalPar.FilterEnvelope = new Envelope(pars->FilterEnvelope, basefreq);
-    NoteGlobalPar.FilterLfo = new LFO(pars->FilterLfo, basefreq);
+    NoteGlobalPar.FilterEnvelope = new Envelope(pars->FilterEnvelope, basefreq, synth);
+    NoteGlobalPar.FilterLfo = new LFO(pars->FilterLfo, basefreq, synth);
     NoteGlobalPar.FilterQ = pars->GlobalFilter->getq();
     NoteGlobalPar.FilterFreqTracking=pars->GlobalFilter->getfreqtracking(basefreq);
 
@@ -305,18 +306,18 @@ PADnote::~PADnote()
 inline void PADnote::fadein(float *smps)
 {
     int zerocrossings = 0;
-    for (int i = 1; i < synth->buffersize; ++i)
+    for (int i = 1; i < synth->p_buffersize; ++i)
         if (smps[i - 1] < 0.0 && smps[i] > 0.0)
             zerocrossings++; // this is only the possitive crossings
 
-    float tmp = (synth->buffersize_f - 1.0) / (zerocrossings + 1) / 3.0;
+    float tmp = (synth->p_buffersize_f - 1.0) / (zerocrossings + 1) / 3.0;
     if (tmp < 8.0)
         tmp = 8.0;
 
     // F2I(tmp, n); // how many samples is the fade-in
-    int n = (tmp > 0.0f) ? lrintf(tmp) : lrintf(tmp - 1.0f);
-    if (n > synth->buffersize)
-        n = synth->buffersize;
+    int n = (tmp > 0.0f) ? (int)truncf(tmp) : (int)truncf(tmp - 1.0f);
+    if (n > synth->p_buffersize)
+        n = synth->p_buffersize;
     for (int i = 0; i < n; ++i)
     {   // fade-in
         float tmp = 0.5 - cosf((float)i / (float) n * PI) * 0.5;
@@ -375,7 +376,7 @@ int PADnote::Compute_Linear(float *outl, float *outr, int freqhi, float freqlo)
         return 1;
     }
     int size = pars->sample[nsample].size;
-    for (int i = 0; i < synth->buffersize; ++i)
+    for (int i = 0; i < synth->p_buffersize; ++i)
     {
         poshi_l += freqhi;
         poshi_r += freqhi;
@@ -407,7 +408,7 @@ int PADnote::Compute_Cubic(float *outl, float *outr, int freqhi, float freqlo)
     }
     int size = pars->sample[nsample].size;
     float xm1, x0, x1, x2, a, b, c;
-    for (int i = 0; i < synth->buffersize; ++i)
+    for (int i = 0; i < synth->p_buffersize; ++i)
     {
         poshi_l += freqhi;
         poshi_r += freqhi;
@@ -452,8 +453,8 @@ int PADnote::noteout(float *outl,float *outr)
     float *smps = pars->sample[nsample].smp;
     if (smps == NULL)
     {
-        memset(outl, 0, synth->buffersize * sizeof(float));
-        memset(outr, 0, synth->buffersize * sizeof(float));
+        memset(outl, 0, synth->p_buffersize * sizeof(float));
+        memset(outr, 0, synth->p_buffersize * sizeof(float));
         return 1;
     }
     float smpfreq = pars->sample[nsample].basefreq;
@@ -462,7 +463,7 @@ int PADnote::noteout(float *outl,float *outr)
     int freqhi = (int) (floorf(freqrap));
     float freqlo = freqrap - floorf(freqrap);
 
-    if (Runtime.Interpolation)
+    if (synth->getRuntime().Interpolation)
         Compute_Cubic(outl, outr, freqhi, freqlo);
     else
         Compute_Linear(outl, outr, freqhi, freqlo);
@@ -480,7 +481,7 @@ int PADnote::noteout(float *outl,float *outr)
     // Apply the punch
     if (NoteGlobalPar.Punch.Enabled != 0)
     {
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {
             float punchamp =
                 NoteGlobalPar.Punch.initialvalue * NoteGlobalPar.Punch.t + 1.0;
@@ -506,17 +507,17 @@ int PADnote::noteout(float *outl,float *outr)
     if (aboveAmplitudeThreshold(globaloldamplitude,globalnewamplitude))
     {
         // Amplitude Interpolation
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {
             float tmpvol = interpolateAmplitude(globaloldamplitude,
                                                 globalnewamplitude, i,
-                                                synth->buffersize);
+                                                synth->p_buffersize);
             outl[i] *= tmpvol * pangainL;
             outr[i] *= tmpvol * pangainR;
         }
     } else
     {
-        for (int i = 0; i < synth->buffersize; ++i)
+        for (int i = 0; i < synth->p_buffersize; ++i)
         {
             outl[i] *= globalnewamplitude * pangainL;
             outr[i] *= globalnewamplitude * pangainR;
@@ -527,8 +528,8 @@ int PADnote::noteout(float *outl,float *outr)
     if (Legato.silent) { // Silencer
         if (Legato.msg != LM_FadeIn)
         {
-            memset(outl, 0, synth->bufferbytes);
-            memset(outr, 0, synth->bufferbytes);
+            memset(outl, 0, synth->p_bufferbytes);
+            memset(outr, 0, synth->p_bufferbytes);
         }
     }
     switch (Legato.msg)
@@ -536,7 +537,7 @@ int PADnote::noteout(float *outl,float *outr)
         case LM_CatchUp : // Continue the catch-up...
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {   //Yea, could be done without the loop...
                 Legato.decounter--;
                 if (Legato.decounter < 1)
@@ -556,7 +557,7 @@ int PADnote::noteout(float *outl,float *outr)
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
             Legato.silent = false;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {
                 Legato.decounter--;
                 if (Legato.decounter < 1)
@@ -574,12 +575,12 @@ int PADnote::noteout(float *outl,float *outr)
         case LM_FadeOut : // Fade-out, then set the catch-up
             if (Legato.decounter == -10)
                 Legato.decounter = Legato.fade.length;
-            for (int i = 0; i < synth->buffersize; ++i)
+            for (int i = 0; i < synth->p_buffersize; ++i)
             {
                 Legato.decounter--;
                 if (Legato.decounter < 1)
                 {
-                    for (int j = i; j < synth->buffersize; ++j)
+                    for (int j = i; j < synth->p_buffersize; ++j)
                         outl[j] = outr[j] = 0.0;
                     Legato.decounter = -10;
                     Legato.silent = true;
@@ -612,9 +613,9 @@ int PADnote::noteout(float *outl,float *outr)
     // If it does, disable the note
     if (NoteGlobalPar.AmpEnvelope->finished() != 0)
     {
-        for (int i = 0 ; i < synth->buffersize; ++i)
+        for (int i = 0 ; i < synth->p_buffersize; ++i)
         {   // fade-out
-            float tmp = 1.0f - (float)i / synth->buffersize_f;
+            float tmp = 1.0f - (float)i / synth->p_buffersize_f;
             outl[i] *= tmp;
             outr[i] *= tmp;
         }
