@@ -6,7 +6,7 @@
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
     Public License as published by the Free Software Foundation, either
-    version 3 of the License, or (at your option) any later version.
+    version 2 of the License, or (at your option) any later version.
 
     yoshimi is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -18,6 +18,9 @@
 */
 
 #include <iostream>
+#include <stdio.h>
+#include <sys/types.h>
+#include <fcntl.h>
 
 using namespace std;
 
@@ -30,6 +33,8 @@ using namespace std;
 #include <list>
 #include <pthread.h>
 #include <semaphore.h>
+#include <cstdio>
+#include <unistd.h>
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
@@ -37,6 +42,7 @@ using namespace std;
 #include <FL/Fl_PNG_Image.H>
 #include "yoshimi-logo.h"
 
+void mainRegisterAudioPort(SynthEngine *s, int portnum);
 
 map<SynthEngine *, MusicClient *> synthInstances;
 list<string> splashMessages;
@@ -47,6 +53,39 @@ static int globalArgc = 0;
 static char **globalArgv = NULL;
 bool bShowGui = true;
 
+int commandStyle;
+int commandCount;
+char commandChr;
+char commandBuffer[COMMAND_SIZE + 2]; // allow for overcount and terminator
+
+
+bool commandProcess(char chr)
+{
+    if (chr >= 0x20 && chr < 0x7f && commandCount < COMMAND_SIZE)
+    {
+        commandBuffer[commandCount] = chr;
+        printf("%c", chr);
+        ++commandCount;
+    }
+    else if (chr == 0x7f)
+    {
+        if (commandCount > 0)
+        {
+            printf("%c %c", 0x08, 0x08);
+            --commandCount;
+            commandBuffer[commandCount] = 0;
+        }
+    }
+    else
+    {
+        commandBuffer[commandCount] = 0;
+        commandCount = 0;
+        return true;
+    }
+    return false;
+}
+
+
 //Andrew Deryabin: signal handling moved to main from Config Runtime
 //It's only suitable for single instance app support
 static struct sigaction yoshimiSigAction;
@@ -80,9 +119,9 @@ void splashTimeout(void *splashWin)
 static void *mainGuiThread(void *arg)
 {
 
-    for(int i = 0; i < globalArgc; ++i)
+    for (int i = 0; i < globalArgc; ++i)
     {
-        if(!strcmp(globalArgv [i], "-i")
+        if (!strcmp(globalArgv [i], "-i")
            || !strcmp(globalArgv [i], "--no-gui")
            || !strcmp(globalArgv [i], "--help")
            || !strcmp(globalArgv [i], "-?"))
@@ -97,15 +136,38 @@ static void *mainGuiThread(void *arg)
 
     map<SynthEngine *, MusicClient *>::iterator it;
     fl_register_images();
-    Fl_Window winSplash(400, 300, "yoshimi splash screen");
-    Fl_Box box(0, 0, 400,300);
-    //Fl_Pixmap pix(yoshimi_logo);
+#if (FL_MAJOR_VERSION == 1 && FL_MINOR_VERSION < 3)
+    char *fname = tmpnam(NULL);
+    if (fname)
+    {
+        FILE *f = fopen(fname, "wb");
+        if (f)
+        {
+            fwrite(yoshimi_logo_png, sizeof(yoshimi_logo_png), 1, f);
+            fclose(f);
+        }
+    }
+    Fl_PNG_Image pix(fname);
+    if (fname)
+    unlink(fname);
+#else
     Fl_PNG_Image pix("yoshimi_logo_png", yoshimi_logo_png, sizeof(yoshimi_logo_png));
+#endif
+    
+    const int splashWidth = 411;
+    const int splashHeight = 311;
+    const int textHeight = 20;
+    const int textBorder = 15;
+    
+    Fl_Window winSplash(splashWidth, splashHeight, "yoshimi splash screen");
+    Fl_Box box(0, 0, splashWidth,splashHeight);
+    //Fl_Pixmap pix(yoshimi_logo);
+
     box.image(pix);
-    Fl_Box boxLb(10, 300-30, 400-20, 30);
+    Fl_Box boxLb(textBorder, splashHeight - textHeight * 2, splashWidth - textBorder * 2, textHeight);
     boxLb.box(FL_NO_BOX);
     boxLb.align(FL_ALIGN_CENTER);
-    boxLb.labelsize(20);
+    boxLb.labelsize(textHeight);
     boxLb.labeltype(FL_NORMAL_LABEL);
     boxLb.labelfont(FL_HELVETICA | FL_ITALIC);
     boxLb.label("Yoshimi is starting up");
@@ -114,21 +176,21 @@ static void *mainGuiThread(void *arg)
     winSplash.clear_border();
     winSplash.border(false);
 
-    if(bShowGui)
+    if (bShowGui)
     {
         //o->Rectangle::set(Fl_Monitor::find(0,0),o->w(),o->h(),fltk::ALIGN_CENTER);
         winSplash.position((Fl::w() - winSplash.w()) / 2, (Fl::h() - winSplash.h()) / 2);
         winSplash.show();
-        Fl::add_timeout(2.0, splashTimeout, &winSplash);
+        Fl::add_timeout(2.5, splashTimeout, &winSplash);
 
     }
 
     do
     {
-        if(bShowGui)
+        if (bShowGui)
         {
             Fl::wait(0.033333);
-            while(!splashMessages.empty())
+            while (!splashMessages.empty())
             {
                 boxLb.copy_label(splashMessages.front().c_str());
                 splashMessages.pop_front();
@@ -141,17 +203,20 @@ static void *mainGuiThread(void *arg)
 
     while (firstSynth->getRuntime().runSynth)
     {        
-        if(firstSynth->getUniqueId() == 0)
+        if (firstSynth->getUniqueId() == 0)
         {
             firstSynth->getRuntime().signalCheck();
+            if (read(0, &commandChr, 1) > 0)
+                if (commandProcess(commandChr))
+                    firstSynth->DecodeCommands( commandBuffer);//getRuntime().Log(commandBuffer);
         }
 
-        for(it = synthInstances.begin(); it != synthInstances.end(); ++it)
+        for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
         {
             SynthEngine *_synth = it->first;
             MusicClient *_client = it->second;
             _synth->getRuntime().deadObjects->disposeBodies();
-            if(!_synth->getRuntime().runSynth && _synth->getUniqueId() > 0)
+            if (!_synth->getRuntime().runSynth && _synth->getUniqueId() > 0)
             {
                 if (_client)
                 {
@@ -175,7 +240,7 @@ static void *mainGuiThread(void *arg)
                 for (int i = 0; !_synth->getRuntime().LogList.empty() && i < 5; ++i)
                 {
                     MasterUI *guiMaster = _synth->getGuiMaster(false);
-                    if(guiMaster)
+                    if (guiMaster)
                     {
                         guiMaster->Log(_synth->getRuntime().LogList.front());
                         _synth->getRuntime().LogList.pop_front();
@@ -185,26 +250,15 @@ static void *mainGuiThread(void *arg)
         }
 
         // where all the action is ...
-        if(bShowGui)
+        if (bShowGui)
         {
             Fl::wait(0.033333);
-            while(!splashMessages.empty())
+            while (!splashMessages.empty())
             {
                 boxLb.copy_label(splashMessages.front().c_str());
                 splashMessages.pop_front();
             }
-            void *msg = Fl::thread_message();
-            if(msg)
-            {
-                SynthEngine *synth = ((SynthEngine *)msg);
-                MasterUI *guiMaster = synth->getGuiMaster();
-                if(!guiMaster)
-                {
-                    cerr << "Error starting Main UI!" << endl;
-                    return (void *)1;
-                }
-                guiMaster->Init(guiMaster->getSynth()->getWindowTitle().c_str());
-            }
+            GuiThreadMsg::processGuiMessages();
         }
         else
             usleep(33333);
@@ -252,7 +306,7 @@ bool mainCreateNewInstance(unsigned int forceId)
     if (synth->getRuntime().showGui)
     {
         synth->setWindowTitle(musicClient->midiClientName());
-        Fl::awake((void *)synth);
+        GuiThreadMsg::sendMessage(synth, GuiThreadMsg::NewSynthEngine, 0);
     }
 
     synth->getRuntime().StartupReport(musicClient);
@@ -262,11 +316,20 @@ bool mainCreateNewInstance(unsigned int forceId)
     else
         cout << "\nStarted "<< synth->getUniqueId() << "\n";
     synthInstances.insert(std::make_pair<SynthEngine *, MusicClient *>(synth, musicClient));
+    //register jack ports for enabled parts
+    for (int npart = 0; npart < NUM_MIDI_PARTS; ++npart)
+    {
+        if(synth->part [npart]->Penabled)
+        {
+            mainRegisterAudioPort(synth, npart);
+        }
+    }
     return true;
 
 bail_out:
+    fcntl(0, F_SETFL, commandStyle);
     synth->getRuntime().runSynth = false;
-    synth->getRuntime().Log("Yoshimi stages a strategic retreat :-(");
+    synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(");
     if (musicClient)
     {
         musicClient->Close();
@@ -283,6 +346,12 @@ bail_out:
 
 int main(int argc, char *argv[])
 {
+    commandCount = 0;
+    commandStyle = fcntl(0, F_GETFL, 0);
+    fcntl (0, F_SETFL, (commandStyle | O_NDELAY)); 
+    
+    
+    cout << "Yoshimi is starting" << endl; // guaranteed start message
     globalArgc = argc;
     globalArgv = argv;
     bool bExitSuccess = false;    
@@ -293,9 +362,9 @@ int main(int argc, char *argv[])
     sem_t semGui;
     if(sem_init(&semGui, 0, 0) == 0)
     {
-        if(pthread_attr_init(&attr) == 0)
+        if (pthread_attr_init(&attr) == 0)
         {
-            if(pthread_create(&thr, &attr, mainGuiThread, (void *)&semGui) == 0)
+            if (pthread_create(&thr, &attr, mainGuiThread, (void *)&semGui) == 0)
             {
                 guiStarted = true;
             }
@@ -303,7 +372,7 @@ int main(int argc, char *argv[])
         }
     }
 
-    if(!guiStarted)
+    if (!guiStarted)
     {        
         cout << "Yoshimi can't start main gui loop!" << endl;
         goto bail_out;
@@ -311,9 +380,6 @@ int main(int argc, char *argv[])
     sem_wait(&semGui);
     sem_destroy(&semGui);
 
-    //cout << "Yoshimi is starting" << endl;
-    splashMessages.push_back("Starting synth engine...");
-
     if (!mainCreateNewInstance(0))
     {
         goto bail_out;
@@ -347,15 +413,18 @@ int main(int argc, char *argv[])
     cout << "\nGoodbye - Play again soon?\n";
     bExitSuccess = true;
 
-bail_out:    
-    for(it = synthInstances.begin(); it != synthInstances.end(); ++it)
+bail_out:
+    fcntl(0, F_SETFL, commandStyle);
+    if (bShowGui && !bExitSuccess) // this could be done better!
+        usleep(2000000);
+    for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
     {
         SynthEngine *_synth = it->first;
         MusicClient *_client = it->second;
         _synth->getRuntime().runSynth = false;
         if(!bExitSuccess)
         {
-            _synth->getRuntime().Log("Yoshimi stages a strategic retreat :-(");
+            _synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(");
         }
 
         if (_client)
@@ -371,8 +440,20 @@ bail_out:
             delete _synth;
         }
     }
-    if(bExitSuccess)
+    if (bExitSuccess)
         exit(EXIT_SUCCESS);
     else
         exit(EXIT_FAILURE);
 }
+
+void mainRegisterAudioPort(SynthEngine *s, int portnum)
+{
+    if (s && (portnum < NUM_MIDI_PARTS) && (portnum >= 0))
+    {
+        map<SynthEngine *, MusicClient *>::iterator it = synthInstances.find(s);
+        if (it != synthInstances.end())
+        {
+            it->second->registerAudioPort(portnum);
+        }
+    }
+}
